<?php
/*
Plugin Name: I18N Search
Description: Search (I18N enabled!)
Version: 2.0.1
Author: Martin Vlcek
Author URI: http://mvlcek.bplaced.net

Public functions:
  return_i18n_search_results($tags=null, $words=null, $first=0, $max=10, $order=null, $lang=null)
      returns an array of search results, each with the attributes url, parent, title, 
      date (UNIX timestamp), excerpt (if $excerptlength > 0), content (if $excerptlength < 0), score.
      The params are described on the configuration page.
  return_i18n_tags()
      returns an ordered array, where the key is the tag and the value is an array of
      urls (slugs) using that tag. Tags starting with "_" are ignored. 

Display functions:
  get_i18n_search_results($params)
      outputs the search results. params is an associative array and includes keys like 'tags'
      (to search for), 'words' (to search for), 'max' (number of results to display), ...
      If no tags or words are given they are taken from the request parameters 'tags' and 'words'
      A param 'numWords' < 0 will output the whole content of the page.
  get_i18n_tags($params)
      outputs the ordered tags in spans with class "tag" and a font-size proportional to the 
      percentage of pages using it (within the percentage range specified by the $params['minTagSize']
      and $params['maxTagSize']).
      Tags starting with "_" are ignored.
  get_i18n_search_form($params)
      outputs a search form. $params['slug'] is the URL of the result page.
*/

# get correct id for plugin
$thisfile = basename(__FILE__, ".php");

define('I18N_WORD_INDEX', 'i18n_word_index.txt'); 
define('I18N_TAG_INDEX', 'i18n_tag_index.txt');
define('I18N_DATE_INDEX', 'i18n_date_index.txt');
define('I18N_SEARCH_SETTINGS_FILE', 'i18n_search_settings.xml');

# for sorting results:
define('I18N_CONTENT_WEIGHT', 1);
define('I18N_TITLE_WEIGHT', 5);
define('I18N_TAG_WEIGHT', 10);
define('I18N_MAX_RESULTS', 10);
define('I18N_NUM_WORDS', 30);
define('I18N_MIN_TAG_SIZE', 100);
define('I18N_MAX_TAG_SIZE', 250);

# actions and filters
define('I18N_ACTION_INDEX', 'search-index');
define('I18N_FILTER_INDEX_PAGE', 'search-index-page');
define('I18N_FILTER_SEARCH_ITEM', 'search-item');
define('I18N_FILTER_DISPLAY_ITEM', 'search-display');

# register plugin
register_plugin(
  $thisfile, 
  'I18N Search',   
  '2.0.1',     
  'Martin Vlcek',
  'http://mvlcek.bplaced.net', 
  'Search (I18N enabled!)',
  'plugins',
  'i18n_search_configure'  
);

# load i18n texts
require_once(GSPLUGINPATH.'i18n_common/common.php');
if (basename($_SERVER['PHP_SELF']) != 'index.php') { // back end only
  i18n_merge('i18n_search', substr($LANG,0,2));
  i18n_merge('i18n_search', 'en');
}

# activate filter
add_action('changedata-save', 'i18n_clear_search_index'); 
add_action('page-delete', 'i18n_clear_search_index'); // GetSimple 3.0+
add_action('plugins-sidebar', 'createSideMenu', array($thisfile, i18n_r('i18n_search/CONFIGURE'))); 
add_filter('content','i18n_search_replace');
add_action('index-pretemplate','i18n_search_rss');

# workaround for page-delete in GetSimple 2.03:
if (basename($_SERVER['PHP_SELF']) == 'deletefile.php') {
  i18n_clear_search_index();
}

function i18n_clear_search_index() {
  if (file_exists(GSDATAOTHERPATH . I18N_WORD_INDEX)) unlink(GSDATAOTHERPATH . I18N_WORD_INDEX);
  if (file_exists(GSDATAOTHERPATH . I18N_TAG_INDEX)) unlink(GSDATAOTHERPATH . I18N_TAG_INDEX);
  if (file_exists(GSDATAOTHERPATH . I18N_DATE_INDEX)) unlink(GSDATAOTHERPATH . I18N_DATE_INDEX);
}

function i18n_create_search_index() {
  require_once(GSPLUGINPATH.'i18n_search/indexer.class.php');
  I18nSearchIndexer::getInstance()->index();
}   

function &i18n_search($tags=null, $words=null, $order=null, $lang=null) {
  $results = array();
  if (!$tags && isset($_REQUEST['tags'])) $tags = trim($_REQUEST['tags']);
  if (!$words && isset($_REQUEST['words'])) $words = trim($_REQUEST['words']);
  if (!$tags && !$words) return $results;
  if ($tags && !is_array($tags)) $tags = trim($tags) != '' ? preg_split("/\s+/", trim($tags)) : array();
  if ($words && !is_array($words)) $words = trim($words) != '' ? preg_split("/\s+/", trim($words)) : array();
  require_once(GSPLUGINPATH.'i18n_search/searcher.class.php');
  $searcher = new I18nSearcher($tags, $words, $order, $lang);
  $results = $searcher->search();
  return $results;
}

function return_i18n_number_of_results($tags=null, $words=null) {
  $results = i18n_search($tags, $words);
  return $results ? count($results) : 0;
}

function &return_i18n_tags() {
  require_once(GSPLUGINPATH.'i18n_search/searcher.class.php');
  $searcher = new I18nReader();
  $tags = $searcher->tags();
  return $tags;
}

function return_i18n_search_results($tags=null, $words=null, $first=0, $max=10, $order=null, $lang=null) {
  $results = i18n_search($tags, $words, $order, $lang);
  $count = count($results);
  if ($max > 0) $results = array_slice($results, $first, $max);
  return array('totalCount' => $count, 'first' => $first, 'results' => $results);
}

function i18n_search_params($params) {
  global $language, $LANG;
  if (file_exists(GSDATAOTHERPATH.I18N_SEARCH_SETTINGS_FILE)) {
    $data = getXML(GSDATAOTHERPATH.I18N_SEARCH_SETTINGS_FILE);
    if ($data) foreach ($data->children() as $child) {
      if (!array_key_exists($child->getName(), $params)) $params[$child->getName()] = (string) $child;
    }
  }
  if (isset($language)) i18n_search_merge_texts($language, $params);
  if (isset($LANG)) i18n_search_merge_texts(substr($LANG,0,2), $params);
  i18n_search_merge_texts('en', $params);
  return $params;  
}

function i18n_search_merge_texts($lang, &$params) { 
  $i18n = array();
  if (!file_exists(GSPLUGINPATH.'i18n_search/lang/'.$lang.'.php')) return false;
  @include(GSPLUGINPATH.'i18n_search/lang/'.$lang.'.php'); 
  if (count($i18n) > 0) foreach ($i18n as $code => $text) {
    if (!array_key_exists($code, $params)) $params[$code] = $text;
  }
  return true;
}

function i18n_search_replace($content) {
  return preg_replace_callback("/(<p>\s*)?\(%\s*(searchform|searchresults|tags|searchrss)(\s+(?:%[^%\)]|[^%])+)?\s*%\)(\s*<\/p>)?/", 'i18n_search_replace_match',$content);
}

function i18n_search_replace_match($match) {
  global $args;
  $function = $match[2];
  $params = array();
  $paramstr = isset($match[3]) ? html_entity_decode(trim($match[3]), ENT_QUOTES, 'UTF-8') : '';
  while (preg_match('/^([a-zA-Z][a-zA-Z0-9_-]*)[:=]([^"\'\s]*|"[^"]*"|\'[^\']*\')(?:\s|$)/', $paramstr, $pmatch)) {
    $key = $pmatch[1];
    $value = trim($pmatch[2]);
    if (substr($value,0,1) == '"' || substr($value,0,1) == "'") $value = substr($value,1,strlen($value)-2);
    $params[$key] = $value;
    $paramstr = substr($paramstr, strlen($pmatch[0]));
  }
  $replacement = '';
  if (@$match[1] && (!@$match[4] || $function == 'searchrss')) $replacement .= $match[1];
  ob_start();
  if ($function == 'searchform') {
    get_i18n_search_form($params);
  } else if ($function == 'searchresults') {
    get_i18n_search_results($params);
  } else if ($function == 'tags') {
    get_i18n_tags($params);
  } else if ($function == 'searchrss') {
    get_i18n_search_rsslink($params);
  }
  $replacement .= ob_get_contents();
  ob_end_clean();
  if (@$match[4] && (!@$match[1] || $function == 'searchrss')) $replacement .= $match[4];
  return $replacement;
}

function i18n_search_rss() {
  global $content;
  $c = stripslashes(htmlspecialchars_decode($content, ENT_QUOTES));
  if (preg_match_all("/\(%\s*searchrss(\s+(?:%[^%\)]|[^%])+)?\s*%\)/", $c, $matches)) {
    foreach ($matches[1] as $match) {
      $params = array();
      $paramstr = isset($match) ? html_entity_decode(trim($match), ENT_QUOTES, 'UTF-8') : '';
      while (preg_match('/^([a-zA-Z][a-zA-Z_-]*)[:=]([^"\'\s]*|"[^"]*"|\'[^\']*\')(?:\s|$)/', $paramstr, $pmatch)) {
        $key = $pmatch[1];
        $value = trim($pmatch[2]);
        if (substr($value,0,1) == '"' || substr($value,0,1) == "'") $value = substr($value,1,strlen($value)-2);
        $params[$key] = $value;
        $paramstr = substr($paramstr, strlen($pmatch[0]));
      }
      if (@$params['name'] && isset($_GET[$params['name']])) {
        $params = i18n_search_params($params);
        include(GSPLUGINPATH.'i18n_search/rss.php');
        die;
      }
    }
  }
}

function get_i18n_search_rsslink($params) {
  global $url, $parent, $SITEURL;
  if (!@$params['name']) return;
  echo '<a href="'.find_url($url,$parent).'?'.$params['name'].'"><img src="'.$SITEURL.'plugins/i18n_search/images/rss.gif" alt="rss"/> '.htmlspecialchars(@$params['title']).'</a>';
}

function i18n_search_configure() {
  include(GSPLUGINPATH.'i18n_search/configure.php');
}

function get_i18n_search_form($params=null) {
  $params = i18n_search_params(is_array($params) ? $params : array());
  include(GSPLUGINPATH.'i18n_search/searchform.php');
}

function get_i18n_search_results($params=null) {
  $params = i18n_search_params(is_array($params) ? $params : array());
  include(GSPLUGINPATH.'i18n_search/searchresults.php');
}

function get_i18n_tags($params=null) {
  $params = i18n_search_params(is_array($params) ? $params : array());
  $minPercent = array_key_exists('minTagSize',$params) ? (int) $params['minTagSize'] : I18N_MIN_TAG_SIZE;
  $maxPercent = array_key_exists('maxTagSize',$params) ? (int) $params['maxTagSize'] : I18N_MAX_TAG_SIZE;
  get_i18n_tags_impl($minPercent, $maxPercent);
}

function get_i18n_tags_impl($minPercent=100, $maxPercent=250) {
  include(GSPLUGINPATH.'i18n_search/tags.php');
}

